/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.crossover;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.OperatorUtil;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection.IMultiSelectorOperatorModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection.IOperatorSelector;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection.RoundRobinOperatorSelector;
import org.opt4j.core.Genotype;
import org.opt4j.operators.crossover.BasicCrossoverModule;
import org.opt4j.operators.crossover.Crossover;

/**
 * Base class for crossover operator module that enables the registration of pre-constructed
 * operators, which thus can be constructed using a non-default constructor.
 * 
 * @author barner
 */
public abstract class CrossoverOperatorModuleBase extends BasicCrossoverModule
		implements IMultiSelectorOperatorModule<Crossover<?>> {

	/** Default selection strategy for crossover operators: round robin */
	private IOperatorSelector operatorSelector = new RoundRobinOperatorSelector();

	/** {@inheritDoc} */
	@Override
	public void addOperatorInstance(Class<? extends Genotype> genotypeType,
			Crossover<?> crossoverOperator) {
		OperatorUtil.addOperatorInstance(genotypeType, crossoverOperator, operatorSelector,
				binder(), getOperatorTypeLiteral());
	}

	/** {@inheritDoc} */
	@Override
	public void addOperatorType(Class<? extends Genotype> genotypeType,
			Class<Crossover<?>> crossoverOperator) {
		OperatorUtil.addOperatorType(genotypeType, crossoverOperator, operatorSelector, binder(),
				getOperatorTypeLiteral());
	}

	/** {@inheritDoc} */
	@Override
	public IOperatorSelector getOperatorSelector(Class<? extends Genotype> genotypeType) {
		return operatorSelector;
	}
}
