/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.convert;

/**
 * Defines an {@link IConverterModule} that converts (parts of) AF3 constructs to DDD-SE-internal
 * representations such as sub-elements of DSML expressions or the conversion of generic DSML
 * expressions into internal constraints.
 * 
 * @author diewald
 */
public interface IConverterModule<I, O> {

	/**
	 * Converts the given input into if applicable or returns {@code null} otherwise.
	 * 
	 * @param input
	 *            to be converted.
	 * @return {@code O} on success, {@code null} otherwise.
	 * 
	 * @throws Exception
	 *             if the conversion fails (badly) such that DSE cannot proceed successfully
	 *             anymore.
	 */
	O convert(I input) throws Exception;
}
