/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.convert.dsml;

import static org.fortiss.af3.exploration.alg.util.DesignSpaceExplorationModelElementFactory.createFixedPropertyLiteral;
import static org.fortiss.tooling.base.utils.AnnotationUtils.getAnnotation;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.getChildrenWithType;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.convert.IConverterModule;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.dsl_v2.model.arithmetic.IArithmeticTermContainer;
import org.fortiss.af3.exploration.dsl_v2.model.arithmetic.ModelElementPropertyLiteral;
import org.fortiss.af3.exploration.dsl_v2.model.expression.IExpression;
import org.fortiss.af3.exploration.lang.ExpressionVisitorBase;
import org.fortiss.af3.exploration.moea.model.dsmlext.FixedPropertyLiteral;
import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.annotation.AnnotationValueService;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * 
 * @author diewald
 */
public class PropertyToFixedValueLiteralConverter extends ExpressionVisitorBase
		implements IConverterModule<IExpression, IExpression> {

	/**
	 * Caches the replacement {@link FixedPropertyLiteral}s that are inserted after visiting the
	 * complete {@link IExpression}.
	 */
	Map<ModelElementPropertyLiteral, FixedPropertyLiteral<?>> replaceMap = new HashMap<>();

	/** {@inheritDoc} */
	@Override
	public IExpression convert(IExpression input) throws Exception {
		input.accept(this, Object.class);

		for(ModelElementPropertyLiteral meLit : getChildrenWithType(input,
				ModelElementPropertyLiteral.class)) {
			FixedPropertyLiteral<?> fixedLit = replaceMap.get(meLit);
			EObject meLitContainer = meLit.eContainer();
			if(!(meLitContainer instanceof IArithmeticTermContainer)) {
				throw new ExplorationException("Unexpected DSML expression: The " +
						ModelElementPropertyLiteral.class.getSimpleName() + " is not contained" +
						"in a " + IArithmeticTermContainer.class.getSimpleName() + ".");
			}
			((IArithmeticTermContainer)meLitContainer).setTerms(fixedLit);
		}

		return input;
	}

	// public Object visit(Sum sum) {
	// IExpression expr = sum.getExpression();
	// if(expr instanceof Allocation) {
	// Sum sum = DSMLModelElementFactory.createSum(exists.getSet(), predicate,
	// createFixedPropertyLiteral(0));
	// }
	// }

	/** {@inheritDoc} */
	@Override
	public Object visit(ModelElementPropertyLiteral mePropLiteral) {
		Map<IModelElement, Object> valueMap = new HashMap<>();
		for(IModelElement me : mePropLiteral.getSetReference().getEntries()) {
			IAnnotatedSpecification spec = getAnnotation(me, mePropLiteral.getSpecificationType());
			AnnotationEntry annEntry = AnnotationValueService.getInstance().getAnnotationEntry(me);
			// Use the spec's class since we need the concrete implementing class for the call
			// below.
			Object annValue = annEntry.getSpecificationValue(spec.getClass());
			valueMap.put(me, annValue);
		}

		FixedPropertyLiteral<?> fixedLit =
				createFixedPropertyLiteral(valueMap, mePropLiteral.getSetReference());
		replaceMap.put(mePropLiteral, fixedLit);
		return null;
	}
}
