/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator;

import java.math.BigDecimal;
import java.math.BigInteger;

import org.fortiss.af3.exploration.lang.ExplorationExpressionEvaluator;
import org.fortiss.af3.exploration.moea.model.dsmlext.FixedPropertyLiteral;

/**
 * Parser for DSML expressions with DDD-SE-specific extensions. These extensions are added during an
 * conversion from "pure" DSML expressions.
 * 
 * @author diewald
 */
public abstract class ExtendedExplorationExpressionEvaluatorBase
		extends ExplorationExpressionEvaluator implements IExtendedExplorationExpressionVisitor {

	/**
	 * Returns the value associated with the {@link FixedPropertyLiteral}. It is a result from a
	 * conversion of a pure DSML expression.
	 */
	@Override
	public Object visit(FixedPropertyLiteral<?> literal) {
		Object value = literal.getValue(selectedVars.get(literal.getSetReference()));

		if(value instanceof Integer) {
			return BigInteger.valueOf((Integer)value);
		} else if(value instanceof Double) {
			return BigDecimal.valueOf((Double)value);
		}
		return value;
	}
}
