/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.Evaluate;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.EvaluatorInjectorProvider;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.NoEmptyPartitionConstraint;
import org.fortiss.tooling.common.util.LambdaUtils;
import org.opt4j.core.DoubleValue;
import org.opt4j.core.Value;

import com.google.inject.Provides;

/**
 * Adds a penalty for candidate solutions that contain empty partitions.
 * 
 * @author diewald
 */
public class NoEmptyPartitionConstraintEvaluator extends
		ConstraintEvaluatorBase<Double, NoEmptyPartitionConstraint, TaskMappingEncoding<?, ?>> {

	/** Prototype Constructor. */
	public NoEmptyPartitionConstraintEvaluator(
			Class<NoEmptyPartitionConstraint> explorationTargetType) {
		super(explorationTargetType);
	}

	/** Constructor. */
	public NoEmptyPartitionConstraintEvaluator(NoEmptyPartitionConstraint explorationTarget) {
		super(explorationTarget);
	}

	/** Exponential penalty function for empty {@link Partition}s. */
	@Provides
	@Evaluate(NoEmptyPartitionConstraintEvaluator.class)
	public DoubleValue evaluate(FailSilentTaskMappingEncoding phenotype,
			@Genotyped PartitionMappingEncoding virtMapping) throws Exception {
		Collection<Partition> emptyPartitions = LambdaUtils.filter(virtMapping.getRequesters(),
				res -> phenotype.getRequesters(res).isEmpty());
		return new DoubleValue(Math.expm1(emptyPartitions.size()));
	}

	/** {@inheritDoc} */
	@Override
	public boolean validate(ExplorationConstraint<?> expTarget,
			EvaluatorInjectorProvider evalInjProvider, Value<?> value) {
		return value.getDouble() <= 10e-6;
	}
}
